"""OTel attribute name constants for Metaflow log records."""

ATTR_FLOW = "metaflow.flow"
ATTR_RUN_ID = "metaflow.run_id"
ATTR_STEP = "metaflow.step"
ATTR_TASK_ID = "metaflow.task_id"
ATTR_ATTEMPT = "metaflow.attempt"
ATTR_LOG_SOURCE = "metaflow.log_source"

SERVICE_NAME = "metaflow"
